@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -29,54 +29,78 @@ $page['file'] = 'actionconf.php';
 $page['scripts'] = ['multiselect.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
-
-// VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
+// VAR							TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'actionid' =>			[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'isset({form}) && {form} == "update"'],
-	'name' =>				[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({add}) || isset({update})', _('Name')],
-	'eventsource' =>		[T_ZBX_INT, O_OPT, null,
-		IN([EVENT_SOURCE_TRIGGERS, EVENT_SOURCE_DISCOVERY, EVENT_SOURCE_AUTO_REGISTRATION, EVENT_SOURCE_INTERNAL]),
-		null
-	],
-	'evaltype' =>			[T_ZBX_INT, O_OPT, null,
-		IN([CONDITION_EVAL_TYPE_AND_OR, CONDITION_EVAL_TYPE_AND, CONDITION_EVAL_TYPE_OR, CONDITION_EVAL_TYPE_EXPRESSION]),
-		'isset({add}) || isset({update})'],
-	'formula' =>			[T_ZBX_STR, O_OPT, null,   null,		'isset({add}) || isset({update})'],
-	'esc_period' =>			[T_ZBX_INT, O_OPT, null,	BETWEEN(60, 999999), null, _('Default operation step duration')],
-	'status' =>				[T_ZBX_INT, O_OPT, null,	IN([ACTION_STATUS_ENABLED, ACTION_STATUS_DISABLED]), null],
-	'def_shortdata' =>		[T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'],
-	'def_longdata' =>		[T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'],
-	'recovery_msg' =>		[T_ZBX_INT, O_OPT, null,	null,		null],
-	'r_shortdata' =>		[T_ZBX_STR, O_OPT, null,	null,		'isset({recovery_msg}) && (isset({add}) || isset({update}))', _('Recovery subject')],
-	'r_longdata' =>			[T_ZBX_STR, O_OPT, null,	null,		'isset({recovery_msg}) && (isset({add}) || isset({update}))', _('Recovery message')],
-	'g_actionid' =>			[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
-	'conditions' =>			[null,		O_OPT,	null,	null,		null],
-	'new_condition' =>		[null,		O_OPT,	null,	null,		'isset({add_condition})'],
-	'operations' =>			[null,		O_OPT,	null,	null,		'isset({add}) || isset({update})'],
-	'edit_operationid' =>	[T_ZBX_STR, O_OPT,	P_ACT,	null,		null],
-	'new_operation' =>		[null,		O_OPT,	null,	null,		'isset({add_operation})'],
-	'opconditions' =>		[null,		O_OPT,	null,	null,		null],
-	'new_opcondition' =>	[null,		O_OPT,	null,	null,		'isset({add_opcondition})'],
+	'actionid' =>						[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'isset({form}) && {form} == "update"'],
+	'name' =>							[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({add}) || isset({update})',
+											_('Name')
+										],
+	'eventsource' =>					[T_ZBX_INT, O_OPT, null,
+											IN([EVENT_SOURCE_TRIGGERS, EVENT_SOURCE_DISCOVERY,
+												EVENT_SOURCE_AUTO_REGISTRATION, EVENT_SOURCE_INTERNAL
+											]),
+											null
+										],
+	'evaltype' =>						[T_ZBX_INT, O_OPT, null,
+											IN([CONDITION_EVAL_TYPE_AND_OR, CONDITION_EVAL_TYPE_AND,
+												CONDITION_EVAL_TYPE_OR, CONDITION_EVAL_TYPE_EXPRESSION
+											]),
+											'isset({add}) || isset({update})'
+										],
+	'formula' =>						[T_ZBX_STR, O_OPT, null,   null,		'isset({add}) || isset({update})'],
+	'esc_period' =>						[T_ZBX_INT, O_OPT, null,	BETWEEN(60, 999999), null,
+											_('Default operation step duration')
+										],
+	'status' =>							[T_ZBX_INT, O_OPT, null,	IN([ACTION_STATUS_ENABLED, ACTION_STATUS_DISABLED]),
+											null
+										],
+	'def_shortdata' =>					[T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'],
+	'def_longdata' =>					[T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'],
+	'r_shortdata' =>					[T_ZBX_STR, O_OPT, null,	null,		null],
+	'r_longdata' =>						[T_ZBX_STR, O_OPT, null,	null,		null],
+	'g_actionid' =>						[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
+	'conditions' =>						[null,		O_OPT,	null,	null,		null],
+	'new_condition' =>					[null,		O_OPT,	null,	null,		'isset({add_condition})'],
+	'operations' =>						[null,		O_OPT,	null,	null,		null],
+	'edit_operationid' =>				[T_ZBX_STR, O_OPT,	P_ACT,	null,		null],
+	'new_operation' =>					[null,		O_OPT,	null,	null,		null],
+	'recovery_operations' =>			[null,		O_OPT,	null,	null,		null],
+	'edit_recovery_operationid' =>		[T_ZBX_STR, O_OPT,	P_ACT,	null,		null],
+	'new_recovery_operation' =>			[null,		O_OPT,	null,	null,		null],
+	'opconditions' =>					[null,		O_OPT,	null,	null,		null],
+	'new_opcondition' =>				[null,		O_OPT,	null,	null,		'isset({add_opcondition})'],
 	// actions
-	'action' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
-								IN('"action.massdelete","action.massdisable","action.massenable"'),
-								null
-							],
-	'add_condition' =>		[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
-	'cancel_new_condition' => [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null, null],
-	'add_operation' =>		[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
-	'cancel_new_operation' => [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null, null],
-	'add_opcondition' =>	[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
-	'cancel_new_opcondition' => [T_ZBX_STR, O_OPT, P_SYS|P_ACT, null, null],
-	'add' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
-	'update' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
-	'delete' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
-	'cancel' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
-	'form' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
-	'form_refresh' =>		[T_ZBX_INT, O_OPT, null,	null,		null],
+	'action' =>							[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
+											IN('"action.massdelete","action.massdisable","action.massenable"'),
+											null
+										],
+	'add_condition' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'cancel_new_condition' =>			[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'add_operation' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'cancel_new_operation' =>			[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'add_recovery_operation' =>			[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'cancel_new_recovery_operation' =>	[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'add_opcondition' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'cancel_new_opcondition' =>			[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'maintenance_mode' =>				[T_ZBX_STR, O_OPT, null,
+											IN([ACTION_MAINTENANCE_MODE_NORMAL, ACTION_MAINTENANCE_MODE_PAUSE]),
+											null,
+											_('Pause operations while in maintenance')
+										],
+	'add' =>							[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'update' =>							[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'delete' =>							[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'cancel' =>							[T_ZBX_STR, O_OPT, P_SYS,		null,	null],
+	'form' =>							[T_ZBX_STR, O_OPT, P_SYS,		null,	null],
+	'form_refresh' =>					[T_ZBX_INT, O_OPT, null,		null,	null],
+	// filter
+	'filter_set' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'filter_rst' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'filter_name' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
+	'filter_status' =>			[T_ZBX_INT, O_OPT, null,	IN([-1, ACTION_STATUS_ENABLED, ACTION_STATUS_DISABLED]),		null],
 	// sort and sortorder
-	'sort' =>			[T_ZBX_STR, O_OPT, P_SYS, IN('"name","status"'),						null],
-	'sortorder' =>		[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
+	'sort' =>							[T_ZBX_STR, O_OPT, P_SYS, IN('"name","status"'),						null],
+	'sortorder' =>						[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
 ];
 
 $dataValid = check_fields($fields);
@@ -99,12 +123,15 @@ if (isset($_REQUEST['actionid'])) {
 /*
  * Actions
  */
-if (isset($_REQUEST['cancel_new_operation'])) {
+if (hasRequest('cancel_new_operation')) {
 	unset($_REQUEST['new_operation']);
 }
-elseif (isset($_REQUEST['cancel_new_opcondition'])) {
+elseif (hasRequest('cancel_new_opcondition')) {
 	unset($_REQUEST['new_opcondition']);
 }
+elseif (hasRequest('cancel_new_recovery_operation')) {
+	unset($_REQUEST['new_recovery_operation']);
+}
 elseif (hasRequest('add') || hasRequest('update')) {
 	$action = [
 		'name' => getRequest('name'),
@@ -112,10 +139,10 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		'esc_period' => getRequest('esc_period', 0),
 		'def_shortdata' => getRequest('def_shortdata', ''),
 		'def_longdata' => getRequest('def_longdata', ''),
-		'recovery_msg' => getRequest('recovery_msg', 0),
 		'r_shortdata' => getRequest('r_shortdata', ''),
 		'r_longdata' => getRequest('r_longdata', ''),
-		'operations' => getRequest('operations', [])
+		'operations' => getRequest('operations', []),
+		'recovery_operations' => getRequest('recovery_operations', [])
 	];
 
 	foreach ($action['operations'] as &$operation) {
@@ -149,6 +176,14 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	}
 	unset($operation);
 
+	foreach ($action['recovery_operations'] as &$recovery_operation) {
+		if (array_key_exists('opmessage', $recovery_operation)
+				&& !array_key_exists('default_msg', $recovery_operation['opmessage'])) {
+			$recovery_operation['opmessage']['default_msg'] = 0;
+		}
+	}
+	unset($recovery_operation);
+
 	$filter = [
 		'conditions' => getRequest('conditions', []),
 		'evaltype' => getRequest('evaltype')
@@ -166,6 +201,12 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	}
 	$action['filter'] = $filter;
 
+	$eventsource = getRequest('eventsource', CProfile::get('web.actionconf.eventsource', EVENT_SOURCE_TRIGGERS));
+
+	if ($eventsource == EVENT_SOURCE_TRIGGERS) {
+		$action['maintenance_mode'] = getRequest('maintenance_mode', ACTION_MAINTENANCE_MODE_NORMAL);
+	}
+
 	DBstart();
 
 	if (hasRequest('update')) {
@@ -178,9 +219,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		$auditAction = AUDIT_ACTION_UPDATE;
 	}
 	else {
-		$action['eventsource'] = getRequest('eventsource',
-			CProfile::get('web.actionconf.eventsource', EVENT_SOURCE_TRIGGERS)
-		);
+		$action['eventsource'] = $eventsource;
 
 		$result = API::Action()->create($action);
 
@@ -201,7 +240,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	}
 	show_messages($result, $messageSuccess, $messageFailed);
 }
-elseif (isset($_REQUEST['delete']) && isset($_REQUEST['actionid'])) {
+elseif (hasRequest('delete') && hasRequest('actionid')) {
 	$result = API::Action()->delete([getRequest('actionid')]);
 
 	if ($result) {
@@ -210,7 +249,7 @@ elseif (isset($_REQUEST['delete']) && isset($_REQUEST['actionid'])) {
 	}
 	show_messages($result, _('Action deleted'), _('Cannot delete action'));
 }
-elseif (isset($_REQUEST['add_condition']) && isset($_REQUEST['new_condition'])) {
+elseif (hasRequest('add_condition') && hasRequest('new_condition')) {
 	$newCondition = getRequest('new_condition');
 
 	if ($newCondition) {
@@ -274,8 +313,8 @@ elseif (isset($_REQUEST['add_condition']) && isset($_REQUEST['new_condition']))
 		}
 	}
 }
-elseif (isset($_REQUEST['add_opcondition']) && isset($_REQUEST['new_opcondition'])) {
-	$new_opcondition = $_REQUEST['new_opcondition'];
+elseif (hasRequest('add_opcondition') && hasRequest('new_opcondition')) {
+	$new_opcondition = getRequest('new_opcondition');
 
 	try {
 		CAction::validateOperationConditions($new_opcondition);
@@ -296,7 +335,7 @@ elseif (isset($_REQUEST['add_opcondition']) && isset($_REQUEST['new_opcondition'
 		error($e->getMessage());
 	}
 }
-elseif (isset($_REQUEST['add_operation']) && isset($_REQUEST['new_operation'])) {
+elseif (hasRequest('add_operation') && hasRequest('new_operation')) {
 	$new_operation = $_REQUEST['new_operation'];
 	$result = true;
 
@@ -326,6 +365,11 @@ elseif (isset($_REQUEST['add_operation']) && isset($_REQUEST['new_operation']))
 			break;
 	}
 
+	$eventsource = getRequest('eventsource', CProfile::get('web.actionconf.eventsource', EVENT_SOURCE_TRIGGERS));
+
+	$new_operation['recovery'] = ACTION_OPERATION;
+	$new_operation['eventsource'] = $eventsource;
+
 	if (API::Action()->validateOperationsIntegrity($new_operation)) {
 		$_REQUEST['operations'] = getRequest('operations', []);
 
@@ -336,8 +380,10 @@ elseif (isset($_REQUEST['add_operation']) && isset($_REQUEST['new_operation']))
 			OPERATION_TYPE_HOST_DISABLE => 0,
 			OPERATION_TYPE_HOST_INVENTORY => 0
 		];
+
 		if (array_key_exists($new_operation['operationtype'], $uniqOperations)) {
 			$uniqOperations[$new_operation['operationtype']]++;
+
 			foreach ($_REQUEST['operations'] as $operationId => $operation) {
 				if (array_key_exists($operation['operationtype'], $uniqOperations)
 					&& (!array_key_exists('id', $new_operation)
@@ -345,6 +391,7 @@ elseif (isset($_REQUEST['add_operation']) && isset($_REQUEST['new_operation']))
 					$uniqOperations[$operation['operationtype']]++;
 				}
 			}
+
 			if ($uniqOperations[$new_operation['operationtype']] > 1) {
 				$result = false;
 				error(_s('Operation "%s" already exists.', operation_type2str($new_operation['operationtype'])));
@@ -353,10 +400,6 @@ elseif (isset($_REQUEST['add_operation']) && isset($_REQUEST['new_operation']))
 		}
 
 		if ($result) {
-			$eventsource = getRequest('eventsource',
-				CProfile::get('web.actionconf.eventsource', EVENT_SOURCE_TRIGGERS)
-			);
-
 			if (isset($_REQUEST['new_operation']['id'])) {
 				$_REQUEST['operations'][$_REQUEST['new_operation']['id']] = $_REQUEST['new_operation'];
 			}
@@ -370,7 +413,28 @@ elseif (isset($_REQUEST['add_operation']) && isset($_REQUEST['new_operation']))
 		unset($_REQUEST['new_operation']);
 	}
 }
-elseif (isset($_REQUEST['edit_operationid'])) {
+elseif (hasRequest('add_recovery_operation') && hasRequest('new_recovery_operation')) {
+	$new_recovery_operation = getRequest('new_recovery_operation');
+
+	$eventsource = getRequest('eventsource', CProfile::get('web.actionconf.eventsource', EVENT_SOURCE_TRIGGERS));
+
+	$new_recovery_operation['recovery'] = ACTION_RECOVERY_OPERATION;
+	$new_recovery_operation['eventsource'] = $eventsource;
+
+	if (API::Action()->validateOperationsIntegrity($new_recovery_operation)) {
+		$_REQUEST['recovery_operations'] = getRequest('recovery_operations', []);
+
+		if (isset($_REQUEST['new_recovery_operation']['id'])) {
+			$_REQUEST['recovery_operations'][$_REQUEST['new_recovery_operation']['id']] = $_REQUEST['new_recovery_operation'];
+		}
+		else {
+			$_REQUEST['recovery_operations'][] = $_REQUEST['new_recovery_operation'];
+		}
+
+		unset($_REQUEST['new_recovery_operation']);
+	}
+}
+elseif (hasRequest('edit_operationid')) {
 	$_REQUEST['edit_operationid'] = array_keys($_REQUEST['edit_operationid']);
 	$edit_operationid = $_REQUEST['edit_operationid'] = array_pop($_REQUEST['edit_operationid']);
 	$_REQUEST['operations'] = getRequest('operations', []);
@@ -380,6 +444,17 @@ elseif (isset($_REQUEST['edit_operationid'])) {
 		$_REQUEST['new_operation']['id'] = $edit_operationid;
 	}
 }
+elseif (hasRequest('edit_recovery_operationid')) {
+	$_REQUEST['edit_recovery_operationid'] = array_keys($_REQUEST['edit_recovery_operationid']);
+	$edit_recovery_operationid = array_pop($_REQUEST['edit_recovery_operationid']);
+	$_REQUEST['edit_recovery_operationid'] = $edit_recovery_operationid;
+	$_REQUEST['recovery_operations'] = getRequest('recovery_operations', []);
+
+	if (array_key_exists($edit_recovery_operationid, $_REQUEST['recovery_operations'])) {
+		$_REQUEST['new_recovery_operation'] = $_REQUEST['recovery_operations'][$edit_recovery_operationid];
+		$_REQUEST['new_recovery_operation']['id'] = $edit_recovery_operationid;
+	}
+}
 elseif (hasRequest('action') && str_in_array(getRequest('action'), ['action.massenable', 'action.massdisable']) && hasRequest('g_actionid')) {
 	$result = true;
 	$enable = (getRequest('action') == 'action.massenable');
@@ -447,6 +522,7 @@ if (hasRequest('form')) {
 		'actionid' => getRequest('actionid'),
 		'new_condition' => getRequest('new_condition', []),
 		'new_operation' => getRequest('new_operation'),
+		'new_recovery_operation' => getRequest('new_recovery_operation'),
 		'config' => $config
 	];
 
@@ -454,12 +530,18 @@ if (hasRequest('form')) {
 		$data['action'] = API::Action()->get([
 			'actionids' => $data['actionid'],
 			'selectOperations' => API_OUTPUT_EXTEND,
+			'selectRecoveryOperations' => ['operationid', 'actionid', 'operationtype', 'opmessage', 'opmessage_grp',
+				'opmessage_usr', 'opcommand', 'opcommand_hst', 'opcommand_grp'
+			],
 			'selectFilter' => ['formula', 'conditions', 'evaltype'],
 			'output' => API_OUTPUT_EXTEND,
 			'editable' => true
 		]);
 		$data['action'] = reset($data['action']);
 
+		$data['action']['recovery_operations'] = $data['action']['recoveryOperations'];
+		unset($data['action']['recoveryOperations']);
+
 		foreach ($data['action']['operations'] as &$operation) {
 			switch ($operation['operationtype']) {
 				case OPERATION_TYPE_GROUP_ADD:
@@ -501,8 +583,8 @@ if (hasRequest('form')) {
 		$data['action']['name'] = getRequest('name');
 		$data['action']['esc_period'] = getRequest('esc_period', SEC_PER_HOUR);
 		$data['action']['status'] = getRequest('status', hasRequest('form_refresh') ? 1 : 0);
-		$data['action']['recovery_msg'] = getRequest('recovery_msg', 0);
 		$data['action']['operations'] = getRequest('operations', []);
+		$data['action']['recovery_operations'] = getRequest('recovery_operations', []);
 
 		$data['action']['filter']['evaltype'] = getRequest('evaltype');
 		$data['action']['filter']['formula'] = getRequest('formula');
@@ -513,6 +595,10 @@ if (hasRequest('form')) {
 			$data['action']['def_longdata'] = getRequest('def_longdata', '');
 			$data['action']['r_shortdata'] = getRequest('r_shortdata', '');
 			$data['action']['r_longdata'] = getRequest('r_longdata', '');
+
+			if ($data['eventsource'] == EVENT_SOURCE_TRIGGERS) {
+				$data['action']['maintenance_mode'] = getRequest('maintenance_mode', ACTION_MAINTENANCE_MODE_NORMAL);
+			}
 		}
 		else {
 			if ($data['eventsource'] == EVENT_SOURCE_TRIGGERS) {
@@ -520,6 +606,9 @@ if (hasRequest('form')) {
 				$data['action']['def_longdata'] = getRequest('def_longdata', ACTION_DEFAULT_MSG_TRIGGER);
 				$data['action']['r_shortdata'] = getRequest('r_shortdata', ACTION_DEFAULT_SUBJ_TRIGGER);
 				$data['action']['r_longdata'] = getRequest('r_longdata', ACTION_DEFAULT_MSG_TRIGGER);
+				$data['action']['maintenance_mode'] = getRequest('maintenance_mode',
+					hasRequest('form_refresh') ? ACTION_MAINTENANCE_MODE_NORMAL : ACTION_MAINTENANCE_MODE_PAUSE
+				);
 			}
 			elseif ($data['eventsource'] == EVENT_SOURCE_DISCOVERY) {
 				$data['action']['def_shortdata'] = getRequest('def_shortdata', ACTION_DEFAULT_SUBJ_DISCOVERY);
@@ -539,24 +628,16 @@ if (hasRequest('form')) {
 	}
 
 	if (!$data['actionid'] && !hasRequest('form_refresh') && $data['eventsource'] == EVENT_SOURCE_TRIGGERS) {
-		$data['action']['filter']['conditions'] = [
-			[
-				'formulaid' => 'A',
-				'conditiontype' => CONDITION_TYPE_MAINTENANCE,
-				'operator' => CONDITION_OPERATOR_NOT_IN,
-				'value' => ''
-			],
-			[
-				'formulaid' => 'B',
-				'conditiontype' => CONDITION_TYPE_TRIGGER_VALUE,
-				'operator' => CONDITION_OPERATOR_EQUAL,
-				'value' => TRIGGER_VALUE_TRUE
-			]
-		];
+		$data['action']['filter']['conditions'] = [[
+			'formulaid' => 'A',
+			'conditiontype' => CONDITION_TYPE_MAINTENANCE,
+			'operator' => CONDITION_OPERATOR_NOT_IN,
+			'value' => ''
+		]];
 	}
 
 	$data['allowedConditions'] = get_conditions_by_eventsource($data['eventsource']);
-	$data['allowedOperations'] = get_operations_by_eventsource($data['eventsource']);
+	$data['allowedOperations'] = getAllowedOperations($data['eventsource']);
 
 	if (!hasRequest('add_condition')) {
 		$data['action']['filter']['conditions'] = CConditionHelper::sortConditionsByFormulaId(
@@ -564,7 +645,7 @@ if (hasRequest('form')) {
 		);
 	}
 
-	// new condition
+	// New condition.
 	$data['new_condition'] = [
 		'conditiontype' => isset($data['new_condition']['conditiontype']) ? $data['new_condition']['conditiontype'] : CONDITION_TYPE_TRIGGER_NAME,
 		'operator' => isset($data['new_condition']['operator']) ? $data['new_condition']['operator'] : CONDITION_OPERATOR_LIKE,
@@ -575,8 +656,8 @@ if (hasRequest('form')) {
 		$data['new_condition']['conditiontype'] = $data['allowedConditions'][0];
 	}
 
-	// new operation
-	if (!empty($data['new_operation']) && !is_array($data['new_operation'])) {
+	// New operation.
+	if ($data['new_operation'] && !is_array($data['new_operation'])) {
 		$data['new_operation'] = [
 			'operationtype' => 0,
 			'esc_period' => 0,
@@ -644,7 +725,12 @@ if (hasRequest('form')) {
 		}
 	}
 
-	// render view
+	// New recovery operation.
+	if ($data['new_recovery_operation'] && !is_array($data['new_recovery_operation'])) {
+		$data['new_recovery_operation'] = ['operationtype' => 0];
+	}
+
+	// Render view.
 	$actionView = new CView('configuration.action.edit', $data);
 	$actionView->render();
 	$actionView->show();
@@ -656,16 +742,38 @@ else {
 	CProfile::update('web.'.$page['file'].'.sort', $sortField, PROFILE_TYPE_STR);
 	CProfile::update('web.'.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR);
 
+	// filter
+	if (hasRequest('filter_set')) {
+		CProfile::update('web.actionconf.filter_name', getRequest('filter_name', ''), PROFILE_TYPE_STR);
+		CProfile::update('web.actionconf.filter_status', getRequest('filter_status', -1), PROFILE_TYPE_INT);
+	}
+	elseif (hasRequest('filter_rst')) {
+		CProfile::delete('web.actionconf.filter_name');
+		CProfile::delete('web.actionconf.filter_status');
+	}
+
+	$filter = [
+		'name' => CProfile::get('web.actionconf.filter_name', ''),
+		'status' => CProfile::get('web.actionconf.filter_status', -1)
+	];
+
 	$data = [
 		'eventsource' => getRequest('eventsource', CProfile::get('web.actionconf.eventsource', EVENT_SOURCE_TRIGGERS)),
 		'sort' => $sortField,
 		'sortorder' => $sortOrder,
+		'filter' => $filter,
 		'config' => $config
 	];
 
 	$data['actions'] = API::Action()->get([
 		'output' => API_OUTPUT_EXTEND,
-		'filter' => ['eventsource' => [$data['eventsource']]],
+		'search' => [
+			'name' => ($filter['name'] === '') ? null : $filter['name'],
+		],
+		'filter' => [
+			'eventsource' => $data['eventsource'],
+			'status' => ($filter['status'] == -1) ? null : $filter['status']
+		],
 		'selectFilter' => ['formula', 'conditions', 'evaltype'],
 		'selectOperations' => API_OUTPUT_EXTEND,
 		'editable' => true,
@@ -697,8 +805,6 @@ else {
 	}
 	unset($action);
 
-
-
 	// sorting && paging
 	order_result($data['actions'], $sortField, $sortOrder);
 	$data['paging'] = getPagingLine($data['actions'], $sortOrder, new CUrl('actionconf.php'));
