@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2013 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -40,9 +40,6 @@ class DB {
 	const FIELD_TYPE_TEXT = 'text';
 
 	private static $schema = null;
-	private static $nodeId = null;
-	private static $maxNodeId = null;
-	private static $minNodeId = null;
 
 	/**
 	 * @var DbBackend
@@ -84,24 +81,9 @@ class DB {
 		throw new DBException($error, $code);
 	}
 
-	/**
-	 * Initializes nodes.
-	 *
-	 * @static
-	 */
-	public static function init() {
-		global $ZBX_LOCALNODEID;
-
-		if (is_null(self::$nodeId)) {
-			self::$nodeId = get_current_nodeid(false);
-			self::$minNodeId = bcadd(bcmul(self::$nodeId, '100000000000000'), bcmul($ZBX_LOCALNODEID, '100000000000'), 0);
-			self::$maxNodeId = bcadd(self::$minNodeId, '99999999999', 0);
-		}
-	}
-
 	/**
 	 * Reserve ids for primary key of passed table.
-	 * If record for table does not exist or value is out of range, ids record is recreated
+	 * If record for table does not exist or value is out of range, ids record is created
 	 * using maximum id from table or minimum allowed value.
 	 *
 	 * @throw APIException
@@ -113,18 +95,15 @@ class DB {
 	 *
 	 * @return string
 	 */
-	protected static function reserveIds($table, $count) {
+	public static function reserveIds($table, $count) {
 		global $DB;
 
-		self::init();
-
 		$tableSchema = self::getSchema($table);
 		$id_name = $tableSchema['key'];
 
 		$sql = 'SELECT nextid'.
 				' FROM ids'.
-				' WHERE nodeid='.self::$nodeId.
-					' AND table_name='.zbx_dbstr($table).
+				' WHERE table_name='.zbx_dbstr($table).
 					' AND field_name='.zbx_dbstr($id_name);
 
 		// SQLite3 does not support this syntax. Since we are in transaction, it can be ignored.
@@ -133,20 +112,23 @@ class DB {
 		}
 
 		$res = DBfetch(DBselect($sql));
+
 		if ($res) {
 			$maxNextId = bcadd($res['nextid'], $count, 0);
-			if (bccomp($maxNextId, self::$maxNodeId, 0) == 1 || bccomp($maxNextId, self::$minNodeId, 0) == -1) {
+
+			if (bccomp($maxNextId, ZBX_DB_MAX_ID) == 1) {
 				$nextid = self::refreshIds($table, $count);
 			}
 			else {
 				$sql = 'UPDATE ids'.
-						' SET nextid=nextid+'.$count.
-						' WHERE nodeid='.self::$nodeId.
-							' AND table_name='.zbx_dbstr($table).
+						' SET nextid='.$maxNextId.
+						' WHERE table_name='.zbx_dbstr($table).
 							' AND field_name='.zbx_dbstr($id_name);
+
 				if (!DBexecute($sql)) {
 					self::exception(self::DBEXECUTE_ERROR, 'DBEXECUTE_ERROR');
 				}
+
 				$nextid = bcadd($res['nextid'], 1, 0);
 			}
 		}
@@ -159,45 +141,43 @@ class DB {
 
 	/**
 	 * Refresh id record for given table.
-	 * Record is deleted and then created again with value of maximum id from table or minimu allowed.
+	 * Record is deleted and then created again with value of maximum id from table or minimum allowed.
 	 *
 	 * @throw APIException
+	 *
 	 * @static
 	 *
 	 * @param string $table table name
-	 * @param int $count number of ids to reserve
+	 * @param int    $count number of ids to reserve
 	 *
 	 * @return string
 	 */
 	private static function refreshIds($table, $count) {
-		self::init();
-
 		$tableSchema = self::getSchema($table);
 		$id_name = $tableSchema['key'];
 
-		$sql = 'DELETE FROM ids'.
-				' WHERE nodeid='.self::$nodeId.
-				' AND table_name='.zbx_dbstr($table).
-				' AND field_name='.zbx_dbstr($id_name);
+		// when we reach the maximum ID, we try to refresh them to check if any IDs have been freed
+		$sql = 'DELETE FROM ids WHERE table_name='.zbx_dbstr($table).' AND field_name='.zbx_dbstr($id_name);
+
 		if (!DBexecute($sql)) {
 			self::exception(self::DBEXECUTE_ERROR, 'DBEXECUTE_ERROR');
 		}
 
-		$sql = 'SELECT MAX('.$id_name.') AS id'.
-				' FROM '.$table.
-				' WHERE '.$id_name.'>='.self::$minNodeId.
-				' AND '.$id_name.'<='.self::$maxNodeId;
-		$row = DBfetch(DBselect($sql));
+		$row = DBfetch(DBselect('SELECT MAX('.$id_name.') AS id FROM '.$table));
 
-		$nextid = ($row && $row['id']) ? $row['id'] : self::$minNodeId;
+		$nextid = ($row && $row['id']) ? $row['id'] : 0;
 
 		$maxNextId = bcadd($nextid, $count, 0);
-		if (bccomp($maxNextId, self::$maxNodeId, 0) == 1) {
-			self::exception(self::RESERVEIDS_ERROR, __METHOD__.' ID greater than maximum allowed for table "'.$table.'"');
+
+		if (bccomp($maxNextId, ZBX_DB_MAX_ID) == 1) {
+			self::exception(
+				self::RESERVEIDS_ERROR, __METHOD__.' ID greater than maximum allowed for table "'.$table.'"'
+			);
 		}
 
-		$sql = 'INSERT INTO ids (nodeid,table_name,field_name,nextid)'.
-				' VALUES ('.self::$nodeId.','.zbx_dbstr($table).','.zbx_dbstr($id_name).','.$maxNextId.')';
+		$sql = 'INSERT INTO ids (table_name,field_name,nextid)'.
+				' VALUES ('.zbx_dbstr($table).','.zbx_dbstr($id_name).','.$maxNextId.')';
+
 		if (!DBexecute($sql)) {
 			self::exception(self::DBEXECUTE_ERROR, 'DBEXECUTE_ERROR');
 		}
@@ -290,7 +270,7 @@ class DB {
 	public static function getDefaults($table) {
 		$table = self::getSchema($table);
 
-		$defaults = array();
+		$defaults = [];
 		foreach ($table['fields'] as $name => $field) {
 			if (isset($field['default'])) {
 				$defaults[$name] = $field['default'];
@@ -299,6 +279,21 @@ class DB {
 		return $defaults;
 	}
 
+	/**
+	 * Returns the default value of the given field.
+	 *
+	 * @param string $table		name of the table
+	 * @param string $field		name of the field
+	 *
+	 * @return string|null
+	 */
+	public static function getDefault($table, $field) {
+		$table = self::getSchema($table);
+		$field = $table['fields'][$field];
+
+		return isset($field['default']) ? $field['default'] : null;
+	}
+
 	public static function checkValueTypes($table, &$values) {
 		global $DB;
 		$tableSchema = self::getSchema($table);
@@ -331,13 +326,13 @@ class DB {
 			else {
 				switch ($tableSchema['fields'][$field]['type']) {
 					case self::FIELD_TYPE_CHAR:
-						$length = zbx_strlen($values[$field]);
-						$values[$field] = zbx_dbstr($values[$field]);
+						$length = mb_strlen($values[$field]);
 
 						if ($length > $tableSchema['fields'][$field]['length']) {
 							self::exception(self::SCHEMA_ERROR, _s('Value "%1$s" is too long for field "%2$s" - %3$d characters. Allowed length is %4$d characters.',
 								$values[$field], $field, $length, $tableSchema['fields'][$field]['length']));
 						}
+						$values[$field] = zbx_dbstr($values[$field]);
 						break;
 					case self::FIELD_TYPE_ID:
 					case self::FIELD_TYPE_UINT:
@@ -359,15 +354,15 @@ class DB {
 						$values[$field] = zbx_dbstr($values[$field]);
 						break;
 					case self::FIELD_TYPE_TEXT:
-						$length = zbx_strlen($values[$field]);
-						$values[$field] = zbx_dbstr($values[$field]);
+						if ($DB['TYPE'] == ZBX_DB_DB2 || $DB['TYPE'] == ZBX_DB_ORACLE) {
+							$length = mb_strlen($values[$field]);
 
-						if ($DB['TYPE'] == ZBX_DB_DB2) {
 							if ($length > 2048) {
 								self::exception(self::SCHEMA_ERROR, _s('Value "%1$s" is too long for field "%2$s" - %3$d characters. Allowed length is 2048 characters.',
 									$values[$field], $field, $length));
 							}
 						}
+						$values[$field] = zbx_dbstr($values[$field]);
 						break;
 				}
 			}
@@ -385,9 +380,9 @@ class DB {
 	 *
 	 * @return array
 	 */
-	public static function find($tableName, array $criteria = array()) {
+	public static function find($tableName, array $criteria = []) {
 		// build the WHERE part
-		$sqlWhere = array();
+		$sqlWhere = [];
 		foreach ($criteria as $field => $value) {
 			// check if the table has this field
 			if (!self::hasField($tableName, $field)) {
@@ -420,7 +415,7 @@ class DB {
 			return true;
 		}
 
-		$resultIds = array();
+		$resultIds = [];
 
 		if ($getids) {
 			$id = self::reserveIds($table, count($values));
@@ -464,18 +459,16 @@ class DB {
 			return true;
 		}
 
-		$resultIds = array();
+		$resultIds = [];
 
 		$tableSchema = self::getSchema($table);
 		$values = self::addMissingFields($tableSchema, $values);
-		$fields = array_keys(reset($values));
 
 		if ($getids) {
 			$id = self::reserveIds($table, count($values));
-			$fields[] = $tableSchema['key'];
 		}
 
-		$newValues = array();
+		$newValues = [];
 		foreach ($values as $key => $row) {
 			if ($getids) {
 				$resultIds[$key] = $id;
@@ -487,6 +480,8 @@ class DB {
 			$newValues[] = $row;
 		}
 
+		$fields = array_keys(reset($newValues));
+
 		$sql = self::getDbBackend()->createInsertQuery($table, $fields, $newValues);
 
 		if (!DBexecute($sql)) {
@@ -524,22 +519,24 @@ class DB {
 			// set creation
 			$sqlSet = '';
 			foreach ($row['values'] as $field => $value) {
-				$sqlSet .= ' '.$field.'='.$value.',';
+				if ($sqlSet !== '') {
+					$sqlSet .= ',';
+				}
+				$sqlSet .= $field.'='.$value;
 			}
-			$sqlSet = rtrim($sqlSet, ',');
 
 			if (!isset($row['where']) || empty($row['where']) || !is_array($row['where'])) {
 				self::exception(self::DBEXECUTE_ERROR, _s('Cannot perform update statement on table "%1$s" without where condition.', $table));
 			}
 
-			// where condition proccess
-			$sqlWhere = array();
+			// where condition processing
+			$sqlWhere = [];
 			foreach ($row['where'] as $field => $values) {
 				if (!isset($tableSchema['fields'][$field]) || is_null($values)) {
 					self::exception(self::DBEXECUTE_ERROR, _s('Incorrect field "%1$s" name or value in where statement for table "%2$s".', $field, $table));
 				}
 				$values = zbx_toArray($values);
-				sort($values); // sorting ids to prevent deadlocks when two transactions depends from each other
+				sort($values); // sorting ids to prevent deadlocks when two transactions depend on each other
 
 				$sqlWhere[] = dbConditionString($field, $values);
 			}
@@ -566,10 +563,10 @@ class DB {
 	 * @return bool
 	 */
 	public static function updateByPk($tableName, $pk, array $values) {
-		return self::update($tableName, array(
-			'where' => array(self::getPk($tableName) => $pk),
+		return self::update($tableName, [
+			'where' => [self::getPk($tableName) => $pk],
 			'values' => $values
-		));
+		]);
 	}
 
 	/**
@@ -586,7 +583,7 @@ class DB {
 	public static function save($tableName, array $data) {
 		$pk = self::getPk($tableName);
 
-		$newRecords = array();
+		$newRecords = [];
 		foreach ($data as $key => $record) {
 			// if the pk is set - update the record
 			if (isset($record[$pk])) {
@@ -630,11 +627,11 @@ class DB {
 		$pk = self::getPk($tableName);
 		$oldRecords = zbx_toHash($oldRecords, $pk);
 
-		$modifiedRecords = array();
-		foreach ($newRecords as $record) {
+		$modifiedRecords = [];
+		foreach ($newRecords as $key => $record) {
 			// if it's a new or modified record - save it later
 			if (!isset($record[$pk]) || self::recordModified($tableName, $oldRecords[$record[$pk]], $record)) {
-				$modifiedRecords[] = $record;
+				$modifiedRecords[$key] = $record;
 			}
 
 			// remove the existing records from the collection, the remaining ones will be deleted
@@ -646,16 +643,97 @@ class DB {
 		// save modified records
 		if ($modifiedRecords) {
 			$modifiedRecords = self::save($tableName, $modifiedRecords);
+
+			// add the new IDs to the new records
+			foreach ($modifiedRecords as $key => $record) {
+				$newRecords[$key][$pk] = $record[$pk];
+			}
 		}
 
 		// delete remaining records
 		if ($oldRecords) {
-			DB::delete($tableName, array(
+			DB::delete($tableName, [
 				$pk => array_keys($oldRecords)
-			));
+			]);
 		}
 
-		return $modifiedRecords;
+		return $newRecords;
+	}
+
+	/**
+	 * Replaces the records given in $groupedOldRecords with the ones given in $groupedNewRecords.
+	 *
+	 * This method can be used to replace related objects in one-to-many relations. Both old and new records
+	 * must be grouped by the ID of the record they belong to. The records will be matched by position, instead of
+	 * the primary key as in DB::replace(). That is, the first new record will update the first old one, second new
+	 * record - the second old one, etc. Since the records are matched by position, the new records should not contain
+	 * primary keys.
+	 *
+	 * Example 1:
+	 * $old = array(2 => array( array('gitemid' => 1, 'color' => 'FF0000') ));
+	 * $new = array(2 => array( array('color' => '00FF00') ));
+	 * var_dump(DB::replaceByPosition('items', $old, $new));
+	 * // array(array('gitemid' => 1, 'color' => '00FF00'))
+	 *
+	 * The new record updated the old one.
+	 *
+	 * Example 2:
+	 * $old = array(2 => array( array('gitemid' => 1, 'color' => 'FF0000') ));
+	 * $new = array(
+	 *     2 => array(
+	 *         array('color' => '00FF00'),
+	 *         array('color' => '0000FF')
+	 *     )
+	 * );
+	 * var_dump(DB::replaceByPosition('items', $old, $new));
+	 * // array(array('gitemid' => 1, 'color' => '00FF00'), array('gitemid' => 2, 'color' => '0000FF'))
+	 *
+	 * The first record was updated, the second one - created.
+	 *
+	 * Example 3:
+	 * $old = array(
+	 *     2 => array(
+	 *         array('gitemid' => 1, 'color' => 'FF0000'),
+	 *         array('gitemid' => 2, 'color' => '0000FF')
+	 *     )
+	 * );
+	 * $new = array(2 => array( array('color' => '00FF00') ));
+	 * var_dump(DB::replaceByPosition('items', $old, $new));
+	 * // array(array('gitemid' => 1, 'color' => '00FF00'))
+	 *
+	 * The first record was updated, the second one - deleted.
+	 *
+	 * @param string 	$tableName			table to update
+	 * @param array 	$groupedOldRecords	grouped old records
+	 * @param array 	$groupedNewRecords	grouped new records
+	 *
+	 * @return array	array of new records not grouped (!).
+	 */
+	public static function replaceByPosition($tableName, array $groupedOldRecords, array $groupedNewRecords) {
+		$pk = self::getPk($tableName);
+
+		$allOldRecords = [];
+		$allNewRecords = [];
+		foreach ($groupedNewRecords as $key => $newRecords) {
+			// if records exist for the parent object - replace them, otherwise create new records
+			if (isset($groupedOldRecords[$key])) {
+				$oldRecords = $groupedOldRecords[$key];
+
+				// updated the records by position
+				$newRecords = self::mergeRecords($oldRecords, $newRecords, $pk);
+
+				foreach ($oldRecords as $record) {
+					$allOldRecords[] = $record;
+				}
+			}
+
+			foreach ($newRecords as $record) {
+				$allNewRecords[] = $record;
+			}
+		}
+
+		// replace the old records with the new ones
+		return self::replace($tableName, $allOldRecords, $allNewRecords);
 	}
 
 	/**
@@ -670,7 +748,9 @@ class DB {
 	 */
 	public static function recordModified($tableName, array $oldRecord, array $newRecord) {
 		foreach ($oldRecord as $field => $value) {
-			if (self::hasField($tableName, $field) && isset($newRecord[$field]) && $value != $newRecord[$field]) {
+			if (self::hasField($tableName, $field)
+					&& isset($newRecord[$field])
+					&& (string) $value !== (string) $newRecord[$field]) {
 				return true;
 			}
 		}
@@ -678,6 +758,30 @@ class DB {
 		return false;
 	}
 
+	/**
+	 * Replace each record in $oldRecords with a corresponding record in $newRecords, but keep the old record IDs.
+	 * The records are match by position, that is, the first new record, replaces the first old record and etc.
+	 * If there are less $newRecords than $oldRecords, the remaining old records will be discarded.
+	 *
+	 * @param array 	$oldRecords		array of old records
+	 * @param array 	$newRecords		array of new records
+	 * @param string 	$pk				name of the private key column
+	 *
+	 * @return array	array of new records with the primary keys from the old ones
+	 */
+	protected static function mergeRecords(array $oldRecords, array $newRecords, $pk) {
+		$result = [];
+		foreach ($newRecords as $i => $record) {
+			if (isset($oldRecords[$i])) {
+				$record[$pk] = $oldRecords[$i][$pk];
+			}
+
+			$result[] = $record;
+		}
+
+		return $result;
+	}
+
 	/**
 	 * Delete data from DB.
 	 *
@@ -700,7 +804,7 @@ class DB {
 		}
 		$table_schema = self::getSchema($table);
 
-		$sqlWhere = array();
+		$sqlWhere = [];
 		foreach ($wheres as $field => $values) {
 			if (!isset($table_schema['fields'][$field]) || is_null($values)) {
 				self::exception(self::DBEXECUTE_ERROR, _s('Incorrect field "%1$s" name or value in where statement for table "%2$s".', $field, $table));
