@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,69 +21,63 @@
 
 class CTableInfo extends CTable {
 
-	public function __construct($message = '...', $class = 'tableinfo') {
-		parent::__construct($message, $class);
-		$this->setOddRowClass('odd_row');
-		$this->setEvenRowClass('even_row');
-		$this->attributes['cellpadding'] = 3;
-		$this->attributes['cellspacing'] = 1;
-		$this->headerClass = 'header';
-		$this->footerClass = 'footer';
+	protected $message;
+
+	public function __construct() {
+		parent::__construct();
+
+		$this->addClass(ZBX_STYLE_LIST_TABLE);
+		$this->setNoDataMessage(_('No data found.'));
 		$this->addMakeVerticalRotationJs = false;
 	}
 
 	public function toString($destroy = true) {
-		$tableId = $this->getAttribute('id');
+		$tableId = $this->getId();
 
 		if(!$tableId) {
 			$tableId = uniqid('t');
-			$this->setAttribute('id', $tableId);
+			$this->setId($tableId);
 		}
 
 		$string = parent::toString($destroy);
 
-		if($this->addMakeVerticalRotationJs) {
+		if ($this->addMakeVerticalRotationJs) {
 			$string .= get_js(
-				'var makeVerticalRotationForTable = function() {
-					var table = jQuery("#'.$tableId.'");
-
-					table.makeVerticalRotation();
-
-					if (IE8) {
-						jQuery(".vertical_rotation_inner", table).css({
-							filter: "progid:DXImageTransform.Microsoft.BasicImage(rotation=2)"
-						});
-					}
-					else if (IE9) {
-						jQuery(".vertical_rotation_inner", table).css({
-							"-ms-transform": "rotate(270deg)"
-						});
-					}
-
-					if (!IE9) {
-						jQuery(".vertical_rotation_inner", table).css({
-							"writing-mode": "tb-rl"
-						});
-					}
-				}
-
-				if(!jQuery.isReady) {
-					jQuery(document).ready(makeVerticalRotationForTable);
-				}
-				else {
-					makeVerticalRotationForTable();
-				}',
+				'var makeVerticalRotationForTable = function() {'."\n".
+				'	jQuery("#'.$tableId.'").makeVerticalRotation();'."\n".
+				'}'."\n".
+				"\n".
+				'if (!jQuery.isReady) {'."\n".
+				'	jQuery(document).ready(makeVerticalRotationForTable);'."\n".
+				'}'."\n".
+				'else {'."\n".
+				'	makeVerticalRotationForTable();'."\n".
+				'}',
 			true);
 		}
-
 		return $string;
 	}
 
+	public function setNoDataMessage($message) {
+		$this->message = $message;
+		return $this;
+	}
+
 	/**
 	 * Rotate table header text vertical.
 	 * Cells must be marked with "vertical_rotation" class.
 	 */
 	public function makeVerticalRotation() {
 		$this->addMakeVerticalRotationJs = true;
+		return $this;
+	}
+
+	public function endToString() {
+		$ret = '';
+		if ($this->rownum == 0 && $this->message !== null) {
+			$ret .= $this->prepareRow(new CCol($this->message), ZBX_STYLE_NOTHING_TO_SHOW)->toString();
+		}
+		$ret .= parent::endToString();
+		return $ret;
 	}
 }
