@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,22 +28,24 @@ class CImportReferencer {
 	/**
 	 * @var array with references to interfaceid (hostid -> reference_name -> interfaceid)
 	 */
-	public $interfacesCache = array();
-	protected $groups = array();
-	protected $templates = array();
-	protected $hosts = array();
-	protected $applications = array();
-	protected $items = array();
-	protected $valueMaps = array();
-	protected $triggers = array();
-	protected $graphs = array();
-	protected $iconMaps = array();
-	protected $maps = array();
-	protected $screens = array();
-	protected $templateScreens = array();
-	protected $macros = array();
-	protected $proxies = array();
-	protected $hostPrototypes = array();
+	public $interfacesCache = [];
+	protected $groups = [];
+	protected $templates = [];
+	protected $hosts = [];
+	protected $applications = [];
+	protected $items = [];
+	protected $valueMaps = [];
+	protected $triggers = [];
+	protected $graphs = [];
+	protected $iconMaps = [];
+	protected $maps = [];
+	protected $screens = [];
+	protected $templateScreens = [];
+	protected $macros = [];
+	protected $proxies = [];
+	protected $hostPrototypes = [];
+	protected $httptests = [];
+	protected $httpsteps = [];
 	protected $groupsRefs;
 	protected $templatesRefs;
 	protected $hostsRefs;
@@ -59,7 +61,8 @@ class CImportReferencer {
 	protected $macrosRefs;
 	protected $proxiesRefs;
 	protected $hostPrototypesRefs;
-
+	protected $httptestsRefs;
+	protected $httpstepsRefs;
 
 	/**
 	 * Get group id by name.
@@ -184,15 +187,20 @@ class CImportReferencer {
 	 *
 	 * @param string $name
 	 * @param string $expression
+	 * @param string $recovery_expression
 	 *
 	 * @return string|bool
 	 */
-	public function resolveTrigger($name, $expression) {
+	public function resolveTrigger($name, $expression, $recovery_expression) {
 		if ($this->triggersRefs === null) {
 			$this->selectTriggers();
 		}
 
-		return isset($this->triggersRefs[$name][$expression]) ? $this->triggersRefs[$name][$expression] : false;
+		return array_key_exists($name, $this->triggersRefs)
+				&& array_key_exists($expression, $this->triggersRefs[$name])
+				&& array_key_exists($recovery_expression, $this->triggersRefs[$name][$expression])
+			? $this->triggersRefs[$name][$expression][$recovery_expression]
+			: false;
 	}
 
 	/**
@@ -299,7 +307,7 @@ class CImportReferencer {
 	 */
 	public function resolveProxy($name) {
 		if ($this->proxiesRefs === null) {
-			$this->selectProxyes();
+			$this->selectProxies();
 		}
 
 		return isset($this->proxiesRefs[$name]) ? $this->proxiesRefs[$name] : false;
@@ -327,6 +335,53 @@ class CImportReferencer {
 		}
 	}
 
+	/**
+	 * Get httptestid by hostid and web scenario name.
+	 *
+	 * @param string $hostid
+	 * @param string $name
+	 *
+	 * @return string|bool
+	 */
+	public function resolveHttpTest($hostid, $name) {
+		if ($this->httptestsRefs === null) {
+			$this->selectHttpTests();
+		}
+
+		return array_key_exists($hostid, $this->httptestsRefs) && array_key_exists($name, $this->httptestsRefs[$hostid])
+			? $this->httptestsRefs[$hostid][$name]
+			: false;
+	}
+
+	/**
+	 * Get httpstepid by hostid, httptestid and web scenario step name.
+	 *
+	 * @param string $hostid
+	 * @param string $httptestid
+	 * @param string $name
+	 *
+	 * @return string|bool
+	 */
+	public function resolveHttpStep($hostid, $httptestid, $name) {
+		if ($this->httpstepsRefs === null) {
+			$this->selectHttpSteps();
+		}
+
+		if (!array_key_exists($hostid, $this->httpstepsRefs)) {
+			return false;
+		}
+
+		if (!array_key_exists($httptestid, $this->httpstepsRefs[$hostid])) {
+			return false;
+		}
+
+		if (!array_key_exists($name, $this->httpstepsRefs[$hostid][$httptestid])) {
+			return false;
+		}
+
+		return $this->httpstepsRefs[$hostid][$httptestid][$name];
+	}
+
 	/**
 	 * Add group names that need association with a database group id.
 	 *
@@ -394,7 +449,7 @@ class CImportReferencer {
 	public function addApplications(array $applications) {
 		foreach ($applications as $host => $apps) {
 			if (!isset($this->applications[$host])) {
-				$this->applications[$host] = array();
+				$this->applications[$host] = [];
 			}
 			$this->applications[$host] = array_unique(array_merge($this->applications[$host], $apps));
 		}
@@ -410,7 +465,7 @@ class CImportReferencer {
 	public function addItems(array $items) {
 		foreach ($items as $host => $keys) {
 			if (!isset($this->items[$host])) {
-				$this->items[$host] = array();
+				$this->items[$host] = [];
 			}
 			$this->items[$host] = array_unique(array_merge($this->items[$host], $keys));
 		}
@@ -428,7 +483,17 @@ class CImportReferencer {
 	}
 
 	/**
-	 * Add value map names that need association with a database value map id.
+	 * Add value map association with valuemap ID.
+	 *
+	 * @param string $name
+	 * @param string $valuemapid
+	 */
+	public function addValueMapRef($name, $valuemapid) {
+		$this->valueMapsRefs[$name] = $valuemapid;
+	}
+
+	/**
+	 * Add value map names that need association with a database value map ID.
 	 *
 	 * @param array $valueMaps
 	 */
@@ -437,18 +502,30 @@ class CImportReferencer {
 	}
 
 	/**
-	 * Add trigger names/expressions that need association with a database trigger id.
-	 * Input array has format:
-	 * array('triggername1' => array('expr1', 'expr2'), 'triggername2' => array('expr1'), ...)
+	 * Add trigger description/expression/recovery_expression that need association with a database trigger id.
 	 *
 	 * @param array $triggers
+	 * @param array $triggers[<description>]
+	 * @param array $triggers[<description>][<expression>]
+	 * @param bool  $triggers[<description>][<expression>][<recovery_expression>]
 	 */
 	public function addTriggers(array $triggers) {
-		foreach ($triggers as $name => $expressions) {
-			if (!isset($this->triggers[$name])) {
-				$this->triggers[$name] = array();
+		foreach ($triggers as $description => $expressions) {
+			if (!array_key_exists($description, $this->triggers)) {
+				$this->triggers[$description] = [];
+			}
+
+			foreach ($expressions as $expression => $recovery_expressions) {
+				if (!array_key_exists($expression, $this->triggers[$description])) {
+					$this->triggers[$description][$expression] = [];
+				}
+
+				foreach ($recovery_expressions as $recovery_expression => $foo) {
+					if (!array_key_exists($recovery_expression, $this->triggers[$description][$expression])) {
+						$this->triggers[$description][$expression][$recovery_expression] = true;
+					}
+				}
 			}
-			$this->triggers[$name] = array_unique(array_merge($this->triggers[$name], $expressions));
 		}
 	}
 
@@ -462,7 +539,7 @@ class CImportReferencer {
 	public function addGraphs(array $graphs) {
 		foreach ($graphs as $host => $hostGraphs) {
 			if (!isset($this->graphs[$host])) {
-				$this->graphs[$host] = array();
+				$this->graphs[$host] = [];
 			}
 			$this->graphs[$host] = array_unique(array_merge($this->graphs[$host], $hostGraphs));
 		}
@@ -473,10 +550,11 @@ class CImportReferencer {
 	 *
 	 * @param string $name
 	 * @param string $expression
-	 * @param string $triggerId
+	 * @param string $recovery_expression
+	 * @param string $triggerid
 	 */
-	public function addTriggerRef($name, $expression, $triggerId) {
-		$this->triggersRefs[$name][$expression] = $triggerId;
+	public function addTriggerRef($name, $expression, $recovery_expression, $triggerid) {
+		$this->triggersRefs[$name][$expression][$recovery_expression] = $triggerid;
 	}
 
 	/**
@@ -553,7 +631,7 @@ class CImportReferencer {
 	public function addMacros(array $macros) {
 		foreach ($macros as $host => $ms) {
 			if (!isset($this->macros[$host])) {
-				$this->macros[$host] = array();
+				$this->macros[$host] = [];
 			}
 			$this->macros[$host] = array_unique(array_merge($this->macros[$host], $ms));
 		}
@@ -597,11 +675,11 @@ class CImportReferencer {
 	public function addHostPrototypes(array $hostPrototypes) {
 		foreach ($hostPrototypes as $host => $discoveryRule) {
 			if (!isset($this->hostPrototypes[$host])) {
-				$this->hostPrototypes[$host] = array();
+				$this->hostPrototypes[$host] = [];
 			}
 			foreach ($discoveryRule as $discoveryRuleKey => $hostPrototypes) {
 				if (!isset($this->hostPrototypes[$host][$discoveryRuleKey])) {
-					$this->hostPrototypes[$host][$discoveryRuleKey] = array();
+					$this->hostPrototypes[$host][$discoveryRuleKey] = [];
 				}
 				$this->hostPrototypes[$host][$discoveryRuleKey] = array_unique(
 					array_merge($this->hostPrototypes[$host][$discoveryRuleKey], $hostPrototypes)
@@ -610,23 +688,61 @@ class CImportReferencer {
 		}
 	}
 
+	/**
+	 * Add web scenario names that need association with a database httptestid.
+	 *
+	 * @param array  $httptests
+	 * @param string $httptests[<host>][]	web scenario name
+	 */
+	public function addHttpTests(array $httptests) {
+		foreach ($httptests as $host => $names) {
+			if (!array_key_exists($host, $this->httptests)) {
+				$this->httptests[$host] = [];
+			}
+
+			$this->httptests[$host] = array_unique(array_merge($this->httptests[$host], $names));
+		}
+	}
+
+	/**
+	 * Add web scenario step names that need association with a database httpstepid.
+	 *
+	 * @param array  $httpsteps
+	 * @param string $httpsteps[<host>][<httptest_name>][]	web scenario step name
+	 */
+	public function addHttpSteps(array $httpsteps) {
+		foreach ($httpsteps as $host => $httptests) {
+			if (!array_key_exists($host, $this->httpsteps)) {
+				$this->httpsteps[$host] = [];
+			}
+
+			foreach ($httptests as $httptest_name => $httpstep_names) {
+				if (!array_key_exists($httptest_name, $this->httpsteps[$host])) {
+					$this->httpsteps[$host][$httptest_name] = [];
+				}
+
+				$this->httpsteps[$host][$httptest_name] =
+					array_unique(array_merge($this->httpsteps[$host][$httptest_name], $httpstep_names));
+			}
+		}
+	}
+
 	/**
 	 * Select group ids for previously added group names.
 	 */
 	protected function selectGroups() {
 		if (!empty($this->groups)) {
-			$this->groupsRefs = array();
-			$dbGroups = API::HostGroup()->get(array(
-				'filter' => array('name' => $this->groups),
-				'output' => array('groupid', 'name'),
-				'preservekeys' => true,
-				'editable' => true
-			));
+			$this->groupsRefs = [];
+			$dbGroups = API::HostGroup()->get([
+				'filter' => ['name' => $this->groups],
+				'output' => ['groupid', 'name'],
+				'preservekeys' => true
+			]);
 			foreach ($dbGroups as $group) {
 				$this->groupsRefs[$group['name']] = $group['groupid'];
 			}
 
-			$this->groups = array();
+			$this->groups = [];
 		}
 	}
 
@@ -635,18 +751,18 @@ class CImportReferencer {
 	 */
 	protected function selectTemplates() {
 		if (!empty($this->templates)) {
-			$this->templatesRefs = array();
-			$dbTemplates = API::Template()->get(array(
-				'output' => array('host', 'templateid'),
+			$this->templatesRefs = [];
+			$dbTemplates = API::Template()->get([
+				'output' => ['host', 'templateid'],
 				'preservekeys' => true,
 				'editable' => true,
-				'filter' => array('host' => $this->templates)
-			));
+				'filter' => ['host' => $this->templates]
+			]);
 			foreach ($dbTemplates as $template) {
 				$this->templatesRefs[$template['host']] = $template['templateid'];
 			}
 
-			$this->templates = array();
+			$this->templates = [];
 		}
 	}
 
@@ -655,20 +771,19 @@ class CImportReferencer {
 	 */
 	protected function selectHosts() {
 		if (!empty($this->hosts)) {
-			$this->hostsRefs = array();
+			$this->hostsRefs = [];
 			// fetch only normal hosts, discovered hosts must not be imported
-			$dbHosts = API::Host()->get(array(
-				'filter' => array('host' => $this->hosts),
-				'output' => array('hostid', 'host'),
+			$dbHosts = API::Host()->get([
+				'filter' => ['host' => $this->hosts],
+				'output' => ['hostid', 'host'],
 				'preservekeys' => true,
-				'templated_hosts' => true,
-				'editable' => true
-			));
+				'templated_hosts' => true
+			]);
 			foreach ($dbHosts as $host) {
 				$this->hostsRefs[$host['host']] = $host['hostid'];
 			}
 
-			$this->hosts = array();
+			$this->hosts = [];
 		}
 	}
 
@@ -677,17 +792,24 @@ class CImportReferencer {
 	 */
 	protected function selectApplications() {
 		if (!empty($this->applications)) {
-			$this->applicationsRefs = array();
-			$sqlWhere = array();
+			$this->applicationsRefs = [];
+			$sqlWhere = [];
+
 			foreach ($this->applications as $host => $applications) {
 				$hostId = $this->resolveHostOrTemplate($host);
 				if ($hostId) {
-					$sqlWhere[] = '(hostid='.zbx_dbstr($hostId).' AND '.dbConditionString('name', $applications).')';
+					$sqlWhere[] = '(a.hostid='.zbx_dbstr($hostId).' AND '.
+						dbConditionString('a.name', $applications).')';
 				}
 			}
 
 			if ($sqlWhere) {
-				$dbApplications = DBselect('SELECT applicationid,hostid,name FROM applications WHERE '.implode(' OR ', $sqlWhere));
+				$dbApplications = DBselect(
+					'SELECT a.applicationid,a.hostid,a.name'.
+					' FROM applications a'.
+					' WHERE '.implode(' OR ', $sqlWhere).
+						' AND a.flags='.ZBX_FLAG_DISCOVERY_NORMAL
+				);
 				while ($dbApplication = DBfetch($dbApplications)) {
 					$this->applicationsRefs[$dbApplication['hostid']][$dbApplication['name']] = $dbApplication['applicationid'];
 				}
@@ -707,9 +829,9 @@ class CImportReferencer {
 	 */
 	protected function selectItems() {
 		if (!empty($this->items)) {
-			$this->itemsRefs = array();
+			$this->itemsRefs = [];
 
-			$sqlWhere = array();
+			$sqlWhere = [];
 			foreach ($this->items as $host => $keys) {
 				$hostId = $this->resolveHostOrTemplate($host);
 				if ($hostId) {
@@ -734,18 +856,22 @@ class CImportReferencer {
 	}
 
 	/**
-	 * Select value map ids for previously added value map names.
+	 * Select value map IDs for previously added value map names.
 	 */
 	protected function selectValueMaps() {
-		if (!empty($this->valueMaps)) {
-			$this->valueMapsRefs = array();
+		if ($this->valueMaps) {
+			$this->valueMapsRefs = [];
 
-			$dbitems = DBselect('SELECT v.name,v.valuemapid FROM valuemaps v WHERE '.dbConditionString('v.name', $this->valueMaps));
-			while ($dbItem = DBfetch($dbitems)) {
-				$this->valueMapsRefs[$dbItem['name']] = $dbItem['valuemapid'];
+			$valuemaps = API::ValueMap()->get([
+				'output' => ['valeumapid', 'name'],
+				'filter' => ['name' => $this->valueMaps]
+			]);
+
+			foreach ($valuemaps as $valuemap) {
+				$this->valueMapsRefs[$valuemap['name']] = $valuemap['valuemapid'];
 			}
 
-			$this->valueMaps = array();
+			$this->valueMaps = [];
 		}
 	}
 
@@ -753,26 +879,34 @@ class CImportReferencer {
 	 * Select trigger ids for previously added trigger names/expressions.
 	 */
 	protected function selectTriggers() {
-		if (!empty($this->triggers)) {
-			$this->triggersRefs = array();
+		if ($this->triggers) {
+			$this->triggersRefs = [];
 
-			$dbTriggers = API::Trigger()->get(array(
-				'output' => array('triggerid', 'expression', 'description'),
-				'expandExpression' => true,
-				'filter' => array(
+			$dbTriggers = API::Trigger()->get([
+				'output' => ['triggerid', 'expression', 'description', 'recovery_expression'],
+				'filter' => [
 					'description' => array_keys($this->triggers),
-					'flags' => array(
+					'flags' => [
 						ZBX_FLAG_DISCOVERY_NORMAL,
 						ZBX_FLAG_DISCOVERY_PROTOTYPE,
 						ZBX_FLAG_DISCOVERY_CREATED
-					)
-				),
-				'editable' => true
-			));
+					]
+				]
+			]);
+
+			$dbTriggers = CMacrosResolverHelper::resolveTriggerExpressions($dbTriggers,
+				['sources' => ['expression', 'recovery_expression']]
+			);
 
 			foreach ($dbTriggers as $dbTrigger) {
-				if (isset($this->triggers[$dbTrigger['description']][$dbTrigger['expression']])) {
-					$this->triggersRefs[$dbTrigger['description']][$dbTrigger['expression']] = $dbTrigger['triggerid'];
+				$description = $dbTrigger['description'];
+				$expression = $dbTrigger['expression'];
+				$recovery_expression = $dbTrigger['recovery_expression'];
+
+				if (array_key_exists($description, $this->triggers)
+						&& array_key_exists($expression, $this->triggers[$description])
+						&& array_key_exists($recovery_expression, $this->triggers[$description][$expression])) {
+					$this->triggersRefs[$description][$expression][$recovery_expression] = $dbTrigger['triggerid'];
 				}
 			}
 		}
@@ -783,9 +917,9 @@ class CImportReferencer {
 	 */
 	protected function selectGraphs() {
 		if ($this->graphs) {
-			$this->graphsRefs = array();
+			$this->graphsRefs = [];
 
-			$graphNames = array();
+			$graphNames = [];
 
 			foreach ($this->graphs as $graphs) {
 				foreach ($graphs as $graph) {
@@ -793,15 +927,14 @@ class CImportReferencer {
 				}
 			}
 
-			$dbGraphs = API::Graph()->get(array(
-				'output' => array('graphid', 'name'),
-				'selectHosts' => array('hostid'),
-				'filter' => array(
+			$dbGraphs = API::Graph()->get([
+				'output' => ['graphid', 'name'],
+				'selectHosts' => ['hostid'],
+				'filter' => [
 					'name' => $graphNames,
 					'flags' => null
-				),
-				'editable' => true
-			));
+				]
+			]);
 
 			foreach ($dbGraphs as $dbGraph) {
 				foreach ($dbGraph['hosts'] as $host) {
@@ -830,17 +963,17 @@ class CImportReferencer {
 	 */
 	protected function selectIconMaps() {
 		if (!empty($this->iconMaps)) {
-			$this->iconMapsRefs = array();
-			$dbIconMaps = API::IconMap()->get(array(
-				'filter' => array('name' => $this->iconMaps),
-				'output' => array('iconmapid', 'name'),
-				'preservekeys' => true,
-			));
+			$this->iconMapsRefs = [];
+			$dbIconMaps = API::IconMap()->get([
+				'filter' => ['name' => $this->iconMaps],
+				'output' => ['iconmapid', 'name'],
+				'preservekeys' => true
+			]);
 			foreach ($dbIconMaps as $iconMap) {
 				$this->iconMapsRefs[$iconMap['name']] = $iconMap['iconmapid'];
 			}
 
-			$this->iconMaps = array();
+			$this->iconMaps = [];
 		}
 	}
 
@@ -849,17 +982,17 @@ class CImportReferencer {
 	 */
 	protected function selectMaps() {
 		if (!empty($this->maps)) {
-			$this->mapsRefs = array();
-			$dbMaps = API::Map()->get(array(
-				'filter' => array('name' => $this->maps),
-				'output' => array('sysmapid', 'name'),
-				'preservekeys' => true,
-			));
+			$this->mapsRefs = [];
+			$dbMaps = API::Map()->get([
+				'filter' => ['name' => $this->maps],
+				'output' => ['sysmapid', 'name'],
+				'preservekeys' => true
+			]);
 			foreach ($dbMaps as $dbMap) {
 				$this->mapsRefs[$dbMap['name']] = $dbMap['sysmapid'];
 			}
 
-			$this->maps = array();
+			$this->maps = [];
 		}
 	}
 
@@ -868,16 +1001,17 @@ class CImportReferencer {
 	 */
 	protected function selectScreens() {
 		if (!empty($this->screens)) {
-			$this->screensRefs = array();
+			$this->screensRefs = [];
 
-			$dbScreens = DBselect('SELECT s.screenid,s.name FROM screens s WHERE'.
-					' s.templateid IS NULL '.
-					' AND '.dbConditionString('s.name', $this->screens));
-			while ($dbScreen = DBfetch($dbScreens)) {
-				$this->screensRefs[$dbScreen['name']] = $dbScreen['screenid'];
+			$db_screens = API::Screen()->get([
+				'filter' => ['name' => $this->screens],
+				'output' => ['screenid', 'name']
+			]);
+			foreach ($db_screens as $db_screen) {
+				$this->screensRefs[$db_screen['name']] = $db_screen['screenid'];
 			}
 
-			$this->screens = array();
+			$this->screens = [];
 		}
 	}
 
@@ -886,20 +1020,17 @@ class CImportReferencer {
 	 */
 	protected function selectTemplateScreens() {
 		if ($this->templateScreens) {
-			$this->templateScreensRefs = array();
+			$this->templateScreensRefs = [];
 
-			$dbScreens = DBselect(
-				'SELECT s.screenid, s.name, s.templateid'.
-				' FROM screens s'.
-				' WHERE s.templateid IS NOT NULL '.
-					' AND '.dbConditionString('s.name', $this->templateScreens)
-			);
-
-			while ($dbScreen = DBfetch($dbScreens)) {
-				$this->templateScreensRefs[$dbScreen['templateid']][$dbScreen['name']] = $dbScreen['screenid'];
+			$db_template_screens = API::TemplateScreen()->get([
+				'filter' => ['name' => $this->templateScreens],
+				'output' => ['screenid', 'name', 'templateid']
+			]);
+			foreach ($db_template_screens as $screen) {
+				$this->templateScreensRefs[$screen['templateid']][$screen['name']] = $screen['screenid'];
 			}
 
-			$this->templateScreens = array();
+			$this->templateScreens = [];
 		}
 	}
 
@@ -908,8 +1039,8 @@ class CImportReferencer {
 	 */
 	protected function selectMacros() {
 		if (!empty($this->macros)) {
-			$this->macrosRefs = array();
-			$sqlWhere = array();
+			$this->macrosRefs = [];
+			$sqlWhere = [];
 			foreach ($this->macros as $host => $macros) {
 				$hostId = $this->resolveHostOrTemplate($host);
 				if ($hostId) {
@@ -924,27 +1055,26 @@ class CImportReferencer {
 				}
 			}
 
-			$this->macros = array();
+			$this->macros = [];
 		}
 	}
 
 	/**
 	 * Select proxy ids for previously added proxy names.
 	 */
-	protected function selectProxyes() {
+	protected function selectProxies() {
 		if (!empty($this->proxies)) {
-			$this->proxiesRefs = array();
-			$dbProxy = API::Proxy()->get(array(
-				'filter' => array('host' => $this->proxies),
-				'output' => array('hostid', 'host'),
-				'preservekeys' => true,
-				'editable' => true
-			));
+			$this->proxiesRefs = [];
+			$dbProxy = API::Proxy()->get([
+				'filter' => ['host' => $this->proxies],
+				'output' => ['hostid', 'host'],
+				'preservekeys' => true
+			]);
 			foreach ($dbProxy as $proxy) {
 				$this->proxiesRefs[$proxy['host']] = $proxy['proxyid'];
 			}
 
-			$this->proxies = array();
+			$this->proxies = [];
 		}
 	}
 
@@ -953,8 +1083,8 @@ class CImportReferencer {
 	 */
 	protected function selectHostPrototypes() {
 		if (!empty($this->hostPrototypes)) {
-			$this->hostPrototypesRefs = array();
-			$sqlWhere = array();
+			$this->hostPrototypesRefs = [];
+			$sqlWhere = [];
 			foreach ($this->hostPrototypes as $host => $discoveryRule) {
 				$hostId = $this->resolveHostOrTemplate($host);
 
@@ -980,4 +1110,80 @@ class CImportReferencer {
 			}
 		}
 	}
+
+	/**
+	 * Select httptestids for previously added web scenario names.
+	 */
+	protected function selectHttpTests() {
+		if ($this->httptests) {
+			$this->httptestsRefs = [];
+
+			$sql_where = [];
+
+			foreach ($this->httptests as $host => $names) {
+				$hostid = $this->resolveHostOrTemplate($host);
+
+				if ($hostid !== false) {
+					$sql_where[] = '(ht.hostid='.zbx_dbstr($hostid).' AND '.dbConditionString('ht.name', $names).')';
+				}
+			}
+
+			if ($sql_where) {
+				$db_httptests = DBselect(
+					'SELECT ht.hostid,ht.name,ht.httptestid'.
+					' FROM httptest ht'.
+					' WHERE '.implode(' OR ', $sql_where)
+				);
+				while ($db_httptest = DBfetch($db_httptests)) {
+					$this->httptestsRefs[$db_httptest['hostid']][$db_httptest['name']] = $db_httptest['httptestid'];
+				}
+			}
+		}
+	}
+
+	/**
+	 * Unset web scenario refs to make referencer select them from db again.
+	 */
+	public function refreshHttpTests() {
+		$this->httptestsRefs = null;
+	}
+
+	/**
+	 * Select httpstepids for previously added web scenario step names.
+	 */
+	protected function selectHttpSteps() {
+		if ($this->httpsteps) {
+			$this->httpstepsRefs = [];
+
+			$sql_where = [];
+
+			foreach ($this->httpsteps as $host => $httptests) {
+				$hostid = $this->resolveHostOrTemplate($host);
+
+				if ($hostid !== false) {
+					foreach ($httptests as $httptest_name => $httpstep_names) {
+						$httptestid = $this->resolveHttpTest($hostid, $httptest_name);
+
+						if ($httptestid !== false) {
+							$sql_where[] = '(hs.httptestid='.zbx_dbstr($httptestid).
+								' AND '.dbConditionString('hs.name', $httpstep_names).')';
+						}
+					}
+				}
+			}
+
+			if ($sql_where) {
+				$db_httpsteps = DBselect(
+					'SELECT ht.hostid,hs.httptestid,hs.name,hs.httpstepid'.
+					' FROM httptest ht,httpstep hs'.
+					' WHERE ht.httptestid=hs.httptestid'.
+						' AND '.implode(' OR ', $sql_where)
+				);
+				while ($db_httpstep = DBfetch($db_httpsteps)) {
+					$this->httpstepsRefs[$db_httpstep['hostid']][$db_httpstep['httptestid']][$db_httpstep['name']] =
+						$db_httpstep['httpstepid'];
+				}
+			}
+		}
+	}
 }
