@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -401,6 +401,7 @@ function zbx_num2bitstr($num, $rev = false) {
 function str2mem($val) {
 	$val = trim($val);
 	$last = strtolower(substr($val, -1));
+	$val = (int) $val;
 
 	switch ($last) {
 		case 'g':
@@ -1261,7 +1262,6 @@ function zbx_toArray($value) {
 		return $value;
 	}
 
-	$result = [];
 	if (is_array($value)) {
 		// reset() is needed to move internal array pointer to the beginning of the array
 		reset($value);
@@ -1272,6 +1272,9 @@ function zbx_toArray($value) {
 		elseif (!empty($value)) {
 			$result = [$value];
 		}
+		else {
+			$result = [];
+		}
 	}
 	else {
 		$result = [$value];
@@ -1285,7 +1288,6 @@ function zbx_objectValues($value, $field) {
 	if (is_null($value)) {
 		return $value;
 	}
-	$result = [];
 
 	if (!is_array($value)) {
 		$result = [$value];
@@ -1294,6 +1296,8 @@ function zbx_objectValues($value, $field) {
 		$result = [$value[$field]];
 	}
 	else {
+		$result = [];
+
 		foreach ($value as $val) {
 			if (!is_array($val)) {
 				$result[] = $val;
@@ -1460,9 +1464,9 @@ function getPageNumber() {
 }
 
 /**
- * Returns paging line.
+ * Returns paging line and recursively slice $items of current page.
  *
- * @param array  $items				list of items
+ * @param array  $items				list of elements
  * @param string $sortorder			the order in which items are sorted ASC or DESC
  * @param CUrl $url					URL object containing arguments and query
  *
@@ -1471,8 +1475,20 @@ function getPageNumber() {
 function getPagingLine(&$items, $sortorder, CUrl $url) {
 	global $page;
 
-	$rowsPerPage = CWebUser::$data['rows_per_page'];
+	$rowsPerPage = (int) CWebUser::$data['rows_per_page'];
+	$config = select_config();
+
 	$itemsCount = count($items);
+	$limit_exceeded = ($config['search_limit'] < $itemsCount);
+	$offset = 0;
+
+	if ($limit_exceeded) {
+		if ($sortorder == ZBX_SORT_DOWN) {
+			$offset = $itemsCount - $config['search_limit'];
+		}
+		$itemsCount = $config['search_limit'];
+	}
+
 	$pagesCount = ($itemsCount > 0) ? ceil($itemsCount / $rowsPerPage) : 1;
 	$currentPage = getPageNumber();
 
@@ -1483,7 +1499,6 @@ function getPagingLine(&$items, $sortorder, CUrl $url) {
 		$currentPage = $pagesCount;
 	}
 
-	$start = ($currentPage - 1) * $rowsPerPage;
 	$tags = [];
 
 	if ($pagesCount > 1) {
@@ -1497,7 +1512,7 @@ function getPagingLine(&$items, $sortorder, CUrl $url) {
 			CProfile::update('web.paging.page', $currentPage, PROFILE_TYPE_INT);
 		}
 
-		// viewed pages (better to use not odd)
+		// viewed pages (better to use odd)
 		$pagingNavRange = 11;
 
 		$endPage = $currentPage + floor($pagingNavRange / 2);
@@ -1512,7 +1527,7 @@ function getPagingLine(&$items, $sortorder, CUrl $url) {
 
 		if ($startPage > 1) {
 			$url->setArgument('page', 1);
-			$tags[] = new CLink(_('First'), $url->getUrl());
+			$tags[] = new CLink(_x('First', 'page navigation'), $url->getUrl());
 		}
 
 		if ($currentPage > 1) {
@@ -1539,38 +1554,27 @@ function getPagingLine(&$items, $sortorder, CUrl $url) {
 
 		if ($p < $pagesCount) {
 			$url->setArgument('page', $pagesCount);
-			$tags[] = new CLink(_('Last'), $url->getUrl());
+			$tags[] = new CLink(_x('Last', 'page navigation'), $url->getUrl());
 		}
 	}
 
+	$total = $limit_exceeded ? $itemsCount.'+' : $itemsCount;
+	$start = ($currentPage - 1) * $rowsPerPage;
+	$end = $start + $rowsPerPage;
+
+	if ($end > $itemsCount) {
+		$end = $itemsCount;
+	}
+
 	if ($pagesCount == 1) {
-		$table_stats = _s('Displaying %1$s of %2$s found', $itemsCount, $itemsCount);
+		$table_stats = _s('Displaying %1$s of %2$s found', $itemsCount, $total);
 	}
 	else {
-		$config = select_config();
-
-		$end = $start + $rowsPerPage;
-		if ($end > $itemsCount) {
-			$end = $itemsCount;
-		}
-		$total = $itemsCount;
-
-		if ($config['search_limit'] < $itemsCount) {
-			if ($sortorder == ZBX_SORT_UP) {
-				array_pop($items);
-			}
-			else {
-				array_shift($items);
-			}
-
-			$total .= '+';
-		}
-
 		$table_stats = _s('Displaying %1$s to %2$s of %3$s found', $start + 1, $end, $total);
 	}
 
-	// trim array with items to contain items for current page
-	$items = array_slice($items, $start, $rowsPerPage, true);
+	// Trim array with elements to contain elements for current page.
+	$items = array_slice($items, $start + $offset, $end - $start, true);
 
 	return (new CDiv())
 		->addClass(ZBX_STYLE_TABLE_PAGING)
@@ -1651,22 +1655,24 @@ function num2letter($number) {
 /**
  * Renders an "access denied" message and stops the execution of the script.
  *
- * The $mode parameters controls the layout of the message:
+ * The $mode parameters controls the layout of the message for logged in users:
  * - ACCESS_DENY_OBJECT     - render the message when denying access to a specific object
  * - ACCESS_DENY_PAGE       - render a complete access denied page
  *
+ * If visitor is without any access permission then layout of the message is same as in ACCESS_DENY_PAGE mode.
+ *
  * @param int $mode
  */
 function access_deny($mode = ACCESS_DENY_OBJECT) {
 	// deny access to an object
-	if ($mode == ACCESS_DENY_OBJECT) {
+	if ($mode == ACCESS_DENY_OBJECT && CWebUser::isLoggedIn()) {
 		require_once dirname(__FILE__).'/page_header.php';
 		show_error_message(_('No permissions to referred object or it does not exist!'));
 		require_once dirname(__FILE__).'/page_footer.php';
 	}
 	// deny access to a page
 	else {
-		// url to redirect the user to after he loggs in
+		// url to redirect the user to after he logs in
 		$url = (new CUrl(!empty($_REQUEST['request']) ? $_REQUEST['request'] : ''))->removeArgument('sid');
 		$url = urlencode($url->toString());
 
@@ -1767,9 +1773,9 @@ function makeMessageBox($good, array $messages, $title = null, $show_close_box =
 	}
 
 	if ($show_close_box) {
-		$msg_box->addItem((new CSpan())
+		$msg_box->addItem((new CSimpleButton())
 			->addClass(ZBX_STYLE_OVERLAY_CLOSE_BTN)
-			->onClick('javascript: $(this).closest(\'.'.$class.'\').remove();')
+			->onClick('jQuery(this).closest(\'.'.$class.'\').remove();')
 			->setAttribute('title', _('Close')));
 	}
 
@@ -2192,19 +2198,6 @@ function hasErrorMesssages() {
 	return false;
 }
 
-/**
- * Check if all keys from $keys exist in $array.
- * If some keys are missing return array of missing keys, true otherwise.
- *
- * @param array $array
- * @param array $keys
- *
- * @return array|bool
- */
-function checkRequiredKeys(array $array, array $keys) {
-	return array_diff($keys, array_keys($array));
-}
-
 /**
  * Clears table rows selection's cookies.
  *
@@ -2301,3 +2294,21 @@ function get_color($image, $color, $alpha = 0) {
 
 	return imagecolorallocate($image, $red, $green, $blue);
 }
+
+/**
+ * Custom error handler for PHP errors.
+ *
+ * @param int     $errno Level of the error raised.
+ * @param string  $errstr Error message.
+ * @param string  $errfile Filename that the error was raised in.
+ * @param int     $errline Line number the error was raised in.
+ */
+function zbx_err_handler($errno, $errstr, $errfile, $errline) {
+	// Necessary to suppress errors when calling with error control operator like @function_name().
+	if (error_reporting() === 0) {
+		return true;
+	}
+
+	// Don't show the call to this handler function.
+	error($errstr.' ['.CProfiler::getInstance()->formatCallStack().']');
+}
