@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -357,28 +357,16 @@ if ($config['event_ack_enable']) {
 		}
 	}
 
-	$problem_events = API::Event()->get([
-		'output' => ['eventid', 'objectid'],
-		'source' => EVENT_SOURCE_TRIGGERS,
-		'object' => EVENT_OBJECT_TRIGGER,
-		'objectids' => $triggerIds,
-		'value' => TRIGGER_VALUE_TRUE,
-		'sortfield' => 'eventid',
-		'sortorder' => 'DESC'
-	]);
+	$problems = getTriggerLastProblems($triggerIds, ['eventid', 'objectid']);
 
-	if ($problem_events) {
-		foreach ($problem_events as $problem_event) {
-			if ($triggers[$problem_event['objectid']]['last_problem_eventid'] == 0) {
-				$triggers[$problem_event['objectid']]['last_problem_eventid'] = $problem_event['eventid'];
-			}
-		}
+	foreach ($problems as $problem) {
+		$triggers[$problem['objectid']]['last_problem_eventid'] = $problem['eventid'];
 	}
-
 }
 
 if ($showEvents == EVENTS_OPTION_ALL || $showEvents == EVENTS_OPTION_NOT_ACK) {
 	foreach ($triggers as &$trigger) {
+		$trigger['display_events'] = false;
 		$trigger['events'] = [];
 	}
 	unset($trigger);
@@ -429,9 +417,22 @@ if ($showEvents == EVENTS_OPTION_ALL || $showEvents == EVENTS_OPTION_NOT_ACK) {
 			}
 
 			$triggers[$event['objectid']]['events'][] = $event;
+
+			if ($showEvents == EVENTS_OPTION_ALL) {
+				$triggers[$event['objectid']]['display_events'] = true;
+			}
+			elseif (!$event['acknowledged']) {
+				$triggers[$event['objectid']]['display_events'] = true;
+			}
 		}
 	}
 }
+else {
+	foreach ($triggers as &$trigger) {
+		$trigger['display_events'] = false;
+	}
+	unset($trigger);
+}
 
 // get trigger dependencies
 $dbTriggerDependencies = DBselect(
@@ -609,7 +610,7 @@ foreach ($triggers as $trigger) {
 	}
 
 	if ($showEvents == EVENTS_OPTION_ALL || $showEvents == EVENTS_OPTION_NOT_ACK) {
-		$openOrCloseButton = $trigger['events']
+		$openOrCloseButton = $trigger['display_events']
 			? (new CSimpleButton())
 				->addClass(ZBX_STYLE_TREEVIEW)
 				->setAttribute('data-switcherid', $trigger['triggerid'])
